using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class CameraController : MonoBehaviour
{
    [Header("Player Distance")] 
    public float TargetDistance = 4;    /* The target distance from the player */
    public float TargetHeight = 2;      /* The target height above the player */
    public float MaxMoveSpeed = 2;      /* The maximum speed the camera can move */
    
    private PlayerController pc;
    
    // Start is called before the first frame update
    void Start()
    {
        pc = GameObject.FindObjectOfType<PlayerController>();
    }

    // Update is called once per frame
    void Update()
    {
        /* Calculate the target position */
        Vector3 target = pc.transform.position - pc.transform.forward * TargetDistance;
        target.y += TargetHeight;
        
        /* Compute the distance from this position */
        float dist = Vector3.Distance(target, transform.position);
        
        /* Create a "scaling factor" to move the camera slower as it approaches the target position */
        float scale = Mathf.Clamp01(dist / 2);
        
        /* Move the camera towards the target position */
        transform.position = Vector3.MoveTowards(transform.position, target, MaxMoveSpeed * scale * Time.deltaTime);
        
        /* Always be facing the player */
        transform.LookAt(pc.transform.position);
    }
}
