using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Powerup : MonoBehaviour
{
    [Header("Cubes")] 
    [SerializeField] private Transform OuterCube;
    [SerializeField] private Transform InnerCube;

    [Header("Settings")] 
    public float Effect = 2;        /* The power of the effect */
    public float Length = 10;       /* The length of time the effect lasts */
    
    // Update is called once per frame
    void Update()
    {
        /* Rotate the cube in a cool way */
        OuterCube.Rotate(new Vector3(50f, 25f, 37f) * Time.deltaTime);
        InnerCube.Rotate(new Vector3(-25f, -12f, -17f) * Time.deltaTime);

    }

    private void OnTriggerEnter(Collider other)
    {
        /* If we collide with the player, we should apply our temporary effect to jump height */
        if (other.gameObject.TryGetComponent(out PlayerController pc))
        {
            pc.StartCoroutine(ApplyEffect(pc));
            Destroy(gameObject);
        }
    }

    IEnumerator ApplyEffect(PlayerController pc)
    {
        /* Capture initial velocity and update */
        float originalVelocity = pc.JumpVelocity;
        pc.JumpVelocity *= Effect;
        
        /* Keep the effect for "Length" seconds */
        float start = Time.time;
        while (Time.time - start < Length)
        {
            yield return null;
        }

        /* Reset the velocity */
        pc.JumpVelocity = originalVelocity;
    }
}
